@@ -19,11 +19,17 @@ class FunctionDefinition(SortableObject):
         eval_globals = math.__dict__
 
         self.name = name
-        args = ', '.join(args)
-        self.function = eval('lambda ' + args + ': ' + function, eval_globals)
+        self.function_string = function
+        self.args = ', '.join(args)
+        self.function = eval('lambda ' + self.args + ': ' + function, eval_globals)
         if self.function is None:
             raise TypeError
 
+
+    def __str__(self):
+        return f"self.name: Args: {self.args}, Expression: {self.function_string}"
+
+
     def sanitized_function(self, species_mappings, parameter_mappings):
         names = sorted(list(species_mappings.keys()) + list(parameter_mappings.keys()), key=lambda x: len(x),
                        reverse=True)
@@ -32,4 +38,4 @@ class FunctionDefinition(SortableObject):
         sanitized_function = self.function
         for id, name in enumerate(names):
             sanitized_function = sanitized_function.replace(name, "{" + str(id) + "}")
-        return sanitized_function.format(*replacements)
\ No newline at end of file
+        return sanitized_function.format(*replacements)
