@@ -44,6 +44,24 @@ def import_SBML(filename, name=None, gillespy_model=None):
     return convert(filename, model_name=name, gillespy_model=gillespy_model)
 
 
+def export_SBML(gillespy_model, filename=None):
+    """
+    GillesPy model to SBML converter
+
+    :param gillespy_model: GillesPy model to be converted to SBML
+    :type gillespy_model: gillespy.Model
+
+    :param filename: Path to the SBML file for conversion
+    :type filename: str
+    """
+    try:
+        from gillespy2.sbml.SBMLexport import export
+    except ImportError:
+        raise ImportError('SBML export conversion not imported successfully')
+
+    return export(gillespy_model, path=filename)
+
+
 class Model(SortableObject):
     # reserved names for model species/parameter names, volume, and operators.
     reserved_names = ['vol']
@@ -159,6 +177,11 @@ class Model(SortableObject):
             print_string += decorate('Rate Rules')
             for rr in sorted(self.listOfRateRules.values()):
                 print_string += '\n' + str(rr)
+        if len(self.listOfFunctionDefinitions):
+            print_string += decorate('Function Definitions')
+            for fd in sorted(self.listOfFunctionDefinitions.values()):
+                print_string += '\n' + str(fd)
+
         return print_string
 
     def serialize(self):
@@ -193,6 +216,16 @@ class Model(SortableObject):
             return ModelError('Name "{}" is unavailable. A species with that name exists.'.format(name))
         if name in self.listOfParameters:
             return ModelError('Name "{}" is unavailable. A parameter with that name exists.'.format(name))
+        if name in self.listOfReactions:
+            return ModelError('Name "{}" is unavailable. A reaction with that name exists.'.format(name))
+        if name in self.listOfEvents:
+            return ModelError('Name "{}" is unavailable. An event with that name exists.'.format(name))
+        if name in self.listOfRateRules:
+            return ModelError('Name "{}" is unavailable. A rate rule with that name exists.'.format(name))
+        if name in self.listOfAssignmentRules:
+            return ModelError('Name "{}" is unavailable. An assignment rule with that name exists.'.format(name))
+        if name in self.listOfFunctionDefinitions:
+            return ModelError('Name "{}" is unavailable. A function definition with that name exists.'.format(name))
         if name.isdigit():
             return ModelError('Name "{}" is unavailable. Names must not be numeric strings.'.format(name))
         for special_character in Model.special_characters:
@@ -396,6 +429,9 @@ class Model(SortableObject):
                 self.add_reaction(r)
         else:
             try:
+                problem = self.problem_with_name(reactions.name)
+                if problem is not None:
+                    raise problem
                 reactions.verify()
                 self.validate_reactants_and_products(reactions)
                 if reactions.name is None or reactions.name == '':
@@ -435,6 +471,9 @@ class Model(SortableObject):
                 self.add_rate_rule(rr)
         else:
             try:
+                problem = self.problem_with_name(rate_rules.name)
+                if problem is not None:
+                    raise problem
                 if len(self.listOfAssignmentRules) != 0:
                     for i in self.listOfAssignmentRules.values():
                         if rate_rules.variable == i.variable:
@@ -476,6 +515,9 @@ class Model(SortableObject):
                 self.add_event(e)
         else:
             try:
+                problem = self.problem_with_name(event.name)
+                if problem is not None:
+                    raise problem
                 if event.trigger is None or not hasattr(event.trigger, 'expression'):
                     raise ModelError(
                         'An Event must contain a valid trigger.')
@@ -499,6 +541,9 @@ class Model(SortableObject):
                 self.add_function_definition(fd)
         else:
             try:
+                problem = self.problem_with_name(function_definitions.name)
+                if problem is not None:
+                    raise problem
                 self.listOfFunctionDefinitions[function_definitions.name] = function_definitions
             except Exception as e:
                 raise ParameterError(
@@ -515,6 +560,9 @@ class Model(SortableObject):
                 self.add_assignment_rule(ar)
         else:
             try:
+                problem = self.problem_with_name(assignment_rules.name)
+                if problem is not None:
+                    raise problem
                 if len(self.listOfRateRules) != 0:
                     for i in self.listOfRateRules.values():
                         if assignment_rules.variable == i.variable:
@@ -538,16 +586,16 @@ class Model(SortableObject):
     def timespan(self, time_span):
         """
         Set the time span of simulation. StochKit does not support non-uniform
-        timespans.
+        timespans. 
 
         :param time_span: Evenly-spaced list of times at which to sample the species
-        populations during the simulation.
+        populations during the simulation. Best to use the form np.linspace(<start time>, <end time>, <number of time-points, inclusive>)
         :type time_span: numpy ndarray
         """
-
-        items = np.diff(time_span)
-        items = np.array([round(item, 10) for item in items])
-        isuniform = (len(set(items)) == 1)
+        
+        first_diff = time_span[1] - time_span[0]
+        other_diff = time_span[2:] - time_span[1:-1]
+        isuniform = np.isclose(other_diff, first_diff).all()
 
         if isuniform:
             self.tspan = time_span
@@ -716,13 +764,12 @@ class Model(SortableObject):
         return 'Element not found!'
 
 
-    def get_best_solver(self, precompile=True):
+    def get_best_solver(self):
         """
         Finds best solver for the users simulation. Currently, AssignmentRules, RateRules, FunctionDefinitions,
         Events, and Species with a dynamic, or continuous population must use the TauHybridSolver.
         :param precompile: If True, and the model contains no AssignmentRules, RateRules, FunctionDefinitions, Events,
-        or Species with a dynamic or continuous population, the get_best_solver will choose the VariableSSACSolver, else
-        it will choose SSACSolver
+        or Species with a dynamic or continuous population, it will choose SSACSolver
         :type precompile: bool
         :return: gillespy2.gillespySolver
         """
@@ -746,21 +793,55 @@ class Model(SortableObject):
             raise ModelError('TauHybridSolver is the only solver currently that supports '
                              'AssignmentRules, RateRules, FunctionDefinitions, or Events. '
                              'Please install Numpy.')
-        from gillespy2.solvers.utilities.cpp_support_test import cpp_support
-
-        if cpp_support is False and can_use_numpy and not hybrid_check:
+        
+        from gillespy2.solvers.cpp import can_use_cpp
+        if can_use_cpp is False and can_use_numpy and not hybrid_check:
             from gillespy2 import NumPySSASolver
             return NumPySSASolver
 
         else:
-            if precompile:
-                from gillespy2 import VariableSSACSolver
-                return VariableSSACSolver
+            from gillespy2 import SSACSolver
+            return SSACSolver
+
+    def get_best_solver_algo(self, algorithm):
+        """
+        If user has specified a particular algorithm, we return either the Python or C++ version of that algorithm
+        """
+        from gillespy2.solvers.cpp import can_use_cpp
+        from gillespy2.solvers.numpy import can_use_numpy
+
+        if not can_use_cpp and can_use_numpy:
+            raise ModelError("Please install C++ or Numpy to use GillesPy2 solvers.")
+
+        if algorithm == 'Tau-Leaping':
+            if can_use_cpp:
+                from gillespy2 import TauLeapingCSolver
+                return TauLeapingCSolver
             else:
+                from gillespy2 import TauLeapingSolver
+                return TauLeapingSolver
+
+        elif algorithm == 'SSA':
+            if can_use_cpp:
                 from gillespy2 import SSACSolver
                 return SSACSolver
+            else:
+                from gillespy2 import NumPySSASolver
+                return NumPySSASolver
 
-    def run(self, solver=None, timeout=0, t=None, show_labels=True, cpp_support=False, **solver_args):
+        elif algorithm == 'ODE':
+            if can_use_cpp:
+                from gillespy2 import ODECSolver
+                return ODECSolver
+            else:
+                from gillespy2 import ODESolver
+                return ODESolver
+        else:
+            raise ModelError("Invalid value for the argument 'algorithm' entered. "
+                             "Please enter 'SSA', 'ODE', or 'Tau-leaping'.")
+
+    def run(self, solver=None, timeout=0, t=None, increment=None, show_labels=True, cpp_support=False, algorithm=None,
+            **solver_args):
         """
         Function calling simulation of the model. There are a number of
         parameters to be set here.
@@ -775,6 +856,7 @@ class Model(SortableObject):
 
         :param t: End time of simulation
         :type t: int
+
         :param solver_args: Solver-specific arguments to be passed to solver.run()
 
         :param cpp_support: INTERNAL USE ONLY, flag for whether or not a computer has the capability to compile a
@@ -785,6 +867,9 @@ class Model(SortableObject):
         True,returns a Results object that inherits UserList and contains one or more Trajectory objects that
         inherit UserDict. Results object supports graphing and csv export.
 
+        :param algorithm: Specify algorithm ('ODE', 'Tau-Leaping', or 'SSA') for GillesPy2 to automatically pick best solver using that algorithm.
+        :type algorithm: str
+
         To pause a simulation and retrieve data before the simulation, keyboard interrupt the simulation by pressing
         control+c or pressing stop on a jupyter notebook. To resume a simulation, pass your previously ran results
         into the run method, and set t = to the time you wish the resuming simulation to end (run(resume=results, t=x)).
@@ -799,17 +884,20 @@ class Model(SortableObject):
             t = self.tspan[-1]
 
         if solver is None:
-            solver = self.get_best_solver()
-
+            if algorithm is not None:
+                solver = self.get_best_solver_algo(algorithm)
+            else:
+                solver = self.get_best_solver()
+        if increment is None:
+            increment = self.tspan[-1] - self.tspan[-2]
         try:
-            solver_results, rc = solver.run(model=self, t=t, increment=self.tspan[-1] - self.tspan[-2],
-                                            timeout=timeout, **solver_args)
+            solver_results, rc = solver.run(model=self, t=t, increment=increment, timeout=timeout, **solver_args)
         except Exception as e:
             # If user has specified the SSACSolver, but they don't actually have a g++ compiler,
             # This will throw an error and throw log. IF a user specifies cpp_support == True and don't have a compiler
             # They would bypass this log.warning and just recieve an error
             if cpp_support is False and not isinstance(solver, str):
-                if solver.name == 'SSACSolver' or solver.name == 'VariableSSACSolver':
+                if solver.name == 'SSACSolver':
                     from gillespy2.core import log
                     log.warning("Please install/configure 'g++' and 'make' on your"
                                 " system, to ensure that GillesPy2 C solvers will"
@@ -835,17 +923,6 @@ class Model(SortableObject):
                 results = results.to_array()
             return results
 
-        if len(solver_results) > 0:
-            results_list = []
-            for i in range(0, len(solver_results)):
-                temp = Trajectory(data=solver_results[i], model=self, solver_name=solver.name, rc=rc)
-                results_list.append(temp)
-
-            results = Results(results_list)
-            if show_labels == False:
-                results = results.to_array()
-            return results
-
         else:
             raise ValueError("number_of_trajectories must be non-negative and non-zero")
 
@@ -1198,7 +1275,7 @@ class StochMLDocument():
 
         for reactant, stoichiometry in R.reactants.items():
             srElement = eTree.Element('SpeciesReference')
-            srElement.set('id', str(reactant))
+            srElement.set('id', str(reactant.name))
             srElement.set('stoichiometry', str(stoichiometry))
             reactants.append(srElement)
 
@@ -1207,7 +1284,7 @@ class StochMLDocument():
         products = eTree.Element('Products')
         for product, stoichiometry in R.products.items():
             srElement = eTree.Element('SpeciesReference')
-            srElement.set('id', str(product))
+            srElement.set('id', str(product.name))
             srElement.set('stoichiometry', str(stoichiometry))
             products.append(srElement)
         e.append(products)
