@@ -1,79 +1,30 @@
-import gillespy2
 from gillespy2.core import gillespyError, GillesPySolver, log
 from gillespy2.solvers.utilities import solverutils as cutils
-import signal, time #for solver timeout implementation
+import signal  # for solver timeout implementation
+import time #for solver timeout implementation
 import os #for getting directories for C++ files
 import shutil #for deleting/copying files
+import threading # for handling read/write to simulation in the background
 import subprocess #For calling make and executing c solver
-import inspect #for finding the Gillespy2 module path
 import tempfile #for temporary directories
-import numpy as np
 
-GILLESPY_PATH = os.path.dirname(inspect.getfile(gillespy2))
-GILLESPY_C_DIRECTORY = os.path.join(GILLESPY_PATH, 'solvers/cpp/c_base')
-MAKE_FILE = os.path.dirname(os.path.abspath(__file__))+'/c_base/makefile'
-
-def _write_constants(outfile, model, reactions, species, parameter_mappings, resume):
-    """
-    This function writes the models constants to a user simulation file
-    :param outfile: CPP file, used for simulating a model
-    :param model: The model that is being simulated
-    :param reactions: List of names of a models reactions
-    :param species: List of sanitized species names
-    :param parameter_mappings: List of sanitized parameter names
-    :param resume: If resuming a simulation from a previous one, resume is the results object from the prior simulation.
-    Else, it is defaulted to None.
-    """
-
-    outfile.write("const double V = {};\n".format(model.volume))
-    outfile.write("std :: string s_names[] = {")
-    if len(species) > 0:
-        # Write model species names.
-        for i in range(len(species)-1):
-            outfile.write('"{}", '.format(species[i]))
-        outfile.write('"{}"'.format(species[-1]))
-        outfile.write("};\nunsigned int populations[] = {")
-        # Write initial populations.
-        for i in range(len(species)-1):
-            # If resuming
-            if not (resume is None):
-                if isinstance(resume, np.ndarray):
-                    outfile.write('{}, '.format(int(resume[0][-1][i+1])))
-                else:
-                    outfile.write('{}, '.format(int(resume[species[i]][-1])))
-            else:
-                outfile.write('{}, '.format(int(model.listOfSpecies[species[i]].initial_value)))
-        if not (resume is None):
-            if isinstance(resume, np.ndarray):
-                outfile.write('{}'.format(int(resume[0][-1][-1])))
-            else:
-                outfile.write('{}'.format(int(resume[species[-1]][-1])))
-        else:
-            outfile.write('{}'.format(int(model.listOfSpecies[species[-1]].initial_value)))
-        outfile.write("};\n")
-    if len(reactions) > 0:
-        # Write reaction names
-        outfile.write("std :: string r_names[] = {")
-        for i in range(len(reactions)-1):
-            outfile.write('"{}", '.format(reactions[i]))
-        outfile.write('"{}"'.format(reactions[-1]))
-        outfile.write("};\n")
-    for param in model.listOfParameters:
-        outfile.write("const double {0} = {1};\n".format(parameter_mappings[param], model.listOfParameters[param].value)
-
-                      )
+GILLESPY_PATH = os.path.dirname(os.path.abspath(__file__))
+GILLESPY_CPP_SSA_DIR = os.path.join(GILLESPY_PATH, 'c_base/ssa_cpp_solver')
+MAKE_FILE = os.path.dirname(os.path.abspath(__file__)) + '/c_base/ssa_cpp_solver/makefile'
+CBASE_DIR = os.path.join(GILLESPY_PATH, 'c_base/')
 
 
 class SSACSolver(GillesPySolver):
     name = "SSACSolver"
-    """TODO"""
 
-    def __init__(self, model=None, output_directory=None, delete_directory=True, resume=None):
+    def __init__(self, model=None, output_directory=None, delete_directory=True, resume=None, variable=True):
         super(SSACSolver, self).__init__()
         self.__compiled = False
         self.delete_directory = False
         self.model = model
         self.resume = resume
+        self.variable = variable
+
         if self.model is not None:
             # Create constant, ordered lists for reactions/species/
             self.species_mappings = self.model.sanitized_species_names()
@@ -98,9 +49,8 @@ class SSACSolver(GillesPySolver):
                 self.output_directory = self.temporary_directory.name
                 
             if not os.path.isdir(self.output_directory):
-                raise gillespyError.DirectoryError("Errors encountered while setting up directory for Solver C++ files."
-                                                   )
-            cutils._copy_files(self.output_directory, GILLESPY_C_DIRECTORY)
+                raise gillespyError.DirectoryError("Errors encountered while setting up directory for Solver C++ files.")
+
             self.__write_template()
             self.__compile()
 
@@ -108,9 +58,15 @@ class SSACSolver(GillesPySolver):
         if self.delete_directory and os.path.isdir(self.output_directory):
             shutil.rmtree(self.output_directory)
         
-    def __write_template(self, template_file='SimulationTemplate.cpp'):
+    def __write_template(self):
         # Open up template file for reading.
-        with open(os.path.join(self.output_directory, template_file), 'r') as template:
+
+        if self.variable:
+            template_file = "VariableSimulationTemplate.cpp"
+        else:
+            template_file = 'SimulationTemplate.cpp'
+
+        with open(os.path.join(GILLESPY_CPP_SSA_DIR, template_file), 'r') as template:
             # Write simulation C++ file.
             template_keyword = "__DEFINE_"
             # Use same lists of model's species and reactions to maintain order
@@ -118,14 +74,17 @@ class SSACSolver(GillesPySolver):
                 for line in template:
                     if line.startswith(template_keyword):
                         line = line[len(template_keyword):]
-                        if line.startswith("CONSTANTS"):
-                            _write_constants(outfile, self.model, self.reactions, self.species, self.parameter_mappings
-                                             ,self.resume)
+                        if line.startswith("VARIABLES"):
+                            cutils.write_variables(outfile, self.model, self.reactions, self.species,
+                                                    self.parameter_mappings, self.resume, variable=self.variable)
                         if line.startswith("PROPENSITY"):
-                            cutils._write_propensity(outfile, self.model, self.species_mappings, self.parameter_mappings
-                                                     , self.reactions)
+                            cutils.write_propensity(outfile, self.model, self.species_mappings, self.parameter_mappings
+                                                    , self.reactions)
                         if line.startswith("REACTIONS"):
-                            cutils._write_reactions(outfile, self.model, self.reactions, self.species)
+                           cutils.write_reactions(outfile, self.model, self.reactions, self.species)
+                        if self.variable:
+                            if line.startswith("PARAMETER_UPDATES"):
+                                cutils.update_parameters(outfile, self.parameters, self.parameter_mappings)
                     else:
                         outfile.write(line)
 
@@ -134,22 +93,13 @@ class SSACSolver(GillesPySolver):
         if self.resume:
             if self.resume[0].model != self.model:
                 raise gillespyError.ModelError('When resuming, one must not alter the model being resumed.')
-            else:
-                built = subprocess.run(["make", "-C", self.output_directory, 'UserSimulation'],
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        else:
-             try:
-                 cleaned = subprocess.run(
-                     ["make", "-C", self.output_directory, '-f', MAKE_FILE,
-                      'cleanSimulation'],
-                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                 built = subprocess.run(
-                     ["make", "-C", self.output_directory, '-f', MAKE_FILE,
-                      'UserSimulation'], stdout=subprocess.PIPE,
-                     stderr=subprocess.PIPE)
-             except KeyboardInterrupt:
-                 log.warning(
-                     "Solver has been interrupted during compile time, unexpected behavior may occur.")
+        try:
+            cmd = ["make", "-C", self.output_directory, '-f', MAKE_FILE, 'UserSimulation',
+                   'GILLESPY_CPP_SSA_DIR=' + GILLESPY_CPP_SSA_DIR, 'CBASE_DIR=' + CBASE_DIR]
+            built = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        except KeyboardInterrupt:
+            log.warning(
+                "Solver has been interrupted during compile time, unexpected behavior may occur.")
 
         if built.returncode == 0:
             self.__compiled = True
@@ -163,12 +113,13 @@ class SSACSolver(GillesPySolver):
         """
         :return: Tuple of strings, denoting all keyword argument for this solvers run() method.
         """
-        return ('model', 't', 'number_of_trajectories', 'timeout', 'increment', 'seed', 'debug', 'profile')
+        return ('model', 't', 'number_of_trajectories', 'timeout', 'increment', 'seed', 'debug', 'profile', 'variables')
 
     def run(self=None, model=None, t=20, number_of_trajectories=1, timeout=0,
-            increment=0.05, seed=None, debug=False, profile=False, resume=None, **kwargs):
+            increment=0.05, seed=None, debug=False, profile=False, variables={}, resume=None, **kwargs):
 
         pause = False
+
         if resume is not None:
             if t < resume['time'][-1]:
                 raise gillespyError.ExecutionError(
@@ -194,18 +145,34 @@ class SSACSolver(GillesPySolver):
                 detected_features.append(feature)
 
         if len(detected_features):
-            raise gillespyError.ModelError('Could not run Model.  SBML Feature: {} not supported by SSACSolver.'
-                                           .format(detected_features))
+                raise gillespyError.ModelError(
+                'Could not run Model.  SBML Feature: {} not supported by SSACSolver.'.format(detected_features))
+
+        if not isinstance(variables, dict):
+            raise gillespyError.SimulationError(
+                'argument to variables must be a dictionary.')
+        for v in variables.keys():
+            if v not in self.species+self.parameters:
+                raise gillespyError.SimulationError('Argument to variable "{}" \
+                is not a valid variable.  Variables must be model species or parameters.'.format(v))
 
         if self.__compiled:
             self.simulation_data = None
             if resume is not None:
-                t = abs(t - resume['time'][-1])
+                t = abs(t - int(resume['time'][-1]))
 
             number_timesteps = int(round(t/increment + 1))
-            # Execute simulation.
-            args = [os.path.join(self.output_directory, 'UserSimulation'), '-trajectories', str(number_of_trajectories),
-                    '-timesteps', str(number_timesteps), '-end', str(t)]
+
+            args = [os.path.join(self.output_directory, 'UserSimulation'),
+                    '-trajectories', str(number_of_trajectories),
+                    '-timesteps', str(number_timesteps),
+                    '-end', str(t)]
+
+            if self.variable:  # Is a variable simulation
+                populations = cutils.update_species_init_values(model.listOfSpecies, self.species, variables, resume)
+                parameter_values = cutils.change_param_values(model.listOfParameters, self.parameters, model.volume, variables)
+                args.extend(['-initial_values', populations, '-parameters', parameter_values])
+
             if seed is not None:
                 if isinstance(seed, int):
                     args.append('-seed')
@@ -218,47 +185,89 @@ class SSACSolver(GillesPySolver):
                     else:
                         raise gillespyError.ModelError("seed must be a positive integer")
 
-            # begin subprocess c simulation with timeout (default timeout=0 will not timeout)
-            with subprocess.Popen(args, stdout=subprocess.PIPE, start_new_session=True) as simulation:
-                return_code = 0
+            # Handler for reading data from subprocess, in background thread.
+            def sim_delegate(sim, sim_buffer):
+                def read_next():
+                    # Reads the next block from the simulation output.
+                    # Returns the length of the string read.
+                    line = sim.stdout.read().decode("utf-8")
+                    ln = len(line)
+                    if ln > 0:
+                        sim_buffer.append(line)
+                    return ln
+                # Read output 1 block at a time, until the program is finished.
+                page_size = read_next()
+                while page_size > 0 and sim.poll() is None:
+                    page_size = read_next()
+
+            # Buffer to store the output of the simulation (retrieved from sim_delegate thread).
+            buffer = []
+            # Each platform is given their own platform-specific sub_kill() function.
+            # Windows event handling
+            if os.name == "nt":
+                sub_kill = lambda sim: sim.send_signal(signal.CTRL_BREAK_EVENT)
+                platform_args = { "creationflags": subprocess.CREATE_NEW_PROCESS_GROUP,
+                                  "start_new_session": True }
+            # POSIX event handling
+            else:
+                sub_kill = lambda sim: os.killpg(sim.pid, signal.SIGINT)
+                platform_args = { "start_new_session": True }
+
+            thread_events = { "timeout": False }
+            with subprocess.Popen(args, stdout=subprocess.PIPE, **platform_args) as simulation:
+                # Put a timer on in the background, if a timeout was specified.
+                def timeout_kill():
+                    thread_events["timeout"] = True
+                    sub_kill(simulation)
+                timeout_thread = threading.Timer(timeout, timeout_kill)
                 try:
+                    output_process = threading.Thread(name="SimulationHandlerThread",
+                                                    target=sim_delegate,
+                                                    args=(simulation, buffer))
+                    output_process.start()
                     if timeout > 0:
-                        stdout, stderr = simulation.communicate(timeout=timeout)
-                    else:
-                        stdout, stderr = simulation.communicate()
-                    return_code = simulation.wait()
+                        timeout_thread.start()
+
+                    # Poll for the program's status; keyboard interrupt is ignored if we use .wait()
+                    while simulation.poll() is None:
+                        time.sleep(0.1)
                 except KeyboardInterrupt:
-                    os.killpg(simulation.pid, signal.SIGINT)  # send signal to the process group
-                    stdout, stderr = simulation.communicate()
+                    sub_kill(simulation)
                     pause = True
-                    return_code = 33
-                except subprocess.TimeoutExpired:
-                        os.killpg(simulation.pid, signal.SIGINT)  # send signal to the process group
-                        stdout, stderr = simulation.communicate()
+                finally:
+                    # Finish off the output reader thread and the timer thread.
+                    return_code = simulation.wait()
+                    output_process.join()
+                    if timeout_thread.is_alive():
+                        timeout_thread.cancel()
+
+                    # Decode from byte, split by comma into array
+                    stdout = "".join(buffer).split(",")
+                    # Check if the simulation had been paused
+                    # (Necessary because we can't set pause to True from thread handler)
+                    if thread_events["timeout"]:
                         pause = True
                         return_code = 33
 
-            # Decode from byte, split by comma into array
-            stdout = stdout.decode('utf-8').split(',')
-            # Parse/return results
-
             if return_code in [0, 33]:
-                trajectory_base, timeStopped = cutils._parse_binary_output(stdout, number_of_trajectories,
-                                                                           number_timesteps, len(model.listOfSpecies), stdout,
-                                                                           pause=pause)
+                trajectory_base, timeStopped = cutils.parse_binary_output(number_of_trajectories, number_timesteps,
+                                                                          len(model.listOfSpecies), stdout, pause=pause)
                 if model.tspan[2] - model.tspan[1] == 1:
                     timeStopped = int(timeStopped)
+
                 # Format results
                 self.simulation_data = []
                 for trajectory in range(number_of_trajectories):
                     data = {'time': trajectory_base[trajectory, :, 0]}
                     for i in range(len(self.species)):
                         data[self.species[i]] = trajectory_base[trajectory, :, i + 1]
+
                     self.simulation_data.append(data)
             else:
                 raise gillespyError.ExecutionError("Error encountered while running simulation C++ file:"
                                                    "\nReturn code: {0}.\nError:\n{1}\n".
                                                    format(simulation.returncode, simulation.stderr))
+
             if resume is not None or timeStopped != 0:
                 self.simulation_data = cutils.c_solver_resume(timeStopped, self.simulation_data, t, resume=resume)
 
