@@ -1,6 +1,6 @@
 import random, math, sys, warnings
 from collections import OrderedDict
-from scipy.integrate import ode, solve_ivp
+from scipy.integrate import ode, LSODA
 import heapq
 import numpy as np
 import threading
@@ -11,7 +11,6 @@ from gillespy2.core.gillespyError import *
 
 eval_globals = math.__dict__
 
-
 def __piecewise(*args):
     # Eval entry for piecewise functions
     args = list(args)
@@ -105,9 +104,12 @@ class TauHybridSolver(GillesPySolver):
         rate_rules = OrderedDict()
 
         # Initialize sample dict
+        rr_vars = {}
+        for n, rr in model.listOfRateRules.items():
+            rr_vars[rr.variable] = n
         for spec in model.listOfSpecies:
-            if spec in model.listOfRateRules:
-                diff_eqs[model.listOfSpecies[spec]] = model.listOfRateRules[spec].formula
+            if spec in rr_vars.keys():
+                diff_eqs[model.listOfSpecies[spec]] = model.listOfRateRules[rr_vars[spec]].formula
             else:
                 diff_eqs[model.listOfSpecies[spec]] = '0'
 
@@ -134,7 +136,6 @@ class TauHybridSolver(GillesPySolver):
         for spec in model.listOfSpecies:
             if diff_eqs[model.listOfSpecies[spec]] == '0':
                 del diff_eqs[model.listOfSpecies[spec]]
-
         # create a dictionary of compiled gillespy2 rate rules
         for spec, rate in diff_eqs.items():
             rate_rules[spec] = compile(gillespy2.RateRule(spec, rate).formula, '<string>', 'eval')
@@ -210,7 +211,6 @@ class TauHybridSolver(GillesPySolver):
         Evaluate the propensities for the reactions and the RHS of the Reactions and RateRules.
         Also evaluates boolean value of event triggers.
         """
-
         state_change = [0] * len(y_map)
         curr_state['t'] = t
         curr_state['time'] = t
@@ -230,7 +230,8 @@ class TauHybridSolver(GillesPySolver):
             state_change[y_map[r]] += propensities[r]
         for event in events:
             triggered = eval(event.trigger.expression, {**eval_globals, **curr_state})
-            if triggered: state_change[y_map[event]] = 1
+            if triggered: 
+                state_change[y_map[event]] = 1
 
         return state_change
 
@@ -414,13 +415,16 @@ class TauHybridSolver(GillesPySolver):
                     event_sensitivity, tau_step, pure_ode):
         """ 
         Helper function to perform the ODE integration of one step.  This
-        method uses scipy.integrate.solve_ivp to get simulation data, and
+        method uses scipy.integrate.LSODA to get simulation data, and
         determines the next stopping point of the simulation. The state is
         updated and returned to __simulate along with curr_time and the
         solution object. 
         """
+        max_step_size = model.tspan[1] - model.tspan[0] / 100
+
         from functools import partial
         events = model.listOfEvents.values()
+        dense_output = False 
         int_args = [curr_state, model.listOfSpecies, model.listOfReactions,
                     model.listOfRateRules,
                     propensities, y_map,
@@ -434,20 +438,30 @@ class TauHybridSolver(GillesPySolver):
         else:
             tau_step = max(1e-6, tau_step)
         if pure_ode:
-            next_tau = model.tspan[-1]
+            next_tau = curr_time+max_step_size
+            dense_output = True
         else:
             next_tau = curr_time + tau_step
         curr_state['t'] = curr_time
         curr_state['time'] = curr_time
 
         # Integrate until end or tau is reached
-        # TODO: Need a way to exit solve_ivp when timeout is triggered
-        sol = solve_ivp(rhs, [curr_time, next_tau], y0,
-                        method=integrator, dense_output=True,
-                        **integrator_options)
-
+        loop_count = 0
+        sol = LSODA(rhs, curr_time, y0, next_tau)
+        counter = 0
+        while sol.t < next_tau:
+            counter += 1
+            sol.step()
+            # Update states of all species based on changes made to species through
+            # ODE processes.  This will update all species whose mode is set to
+            # 'continuous', as well as 'dynamic' mode species which have been
+            # flagged as deterministic.
+            for spec_name, species in model.listOfSpecies.items():
+                if not species.constant:
+                    curr_state[spec_name] = sol.y[y_map[spec_name]]
 
         # Search for precise event times
+        '''
         if len(model.listOfEvents):
             event_times = self.__detect_events(event_sensitivity, sol, model, delayed_events,
                                                trigger_states, curr_time, curr_state)
@@ -456,24 +470,18 @@ class TauHybridSolver(GillesPySolver):
 
         # Get next tau time
         reaction_times = []
-
+        '''
         # Set curr time to next time a change occurs in the system outside of
         # the standard ODE process.  Determine what kind of change this is,
         # and set the curr_time of simulation to restart simulation after
         # making the appropriate state changes.
+        event_times = {}
+        reaction_times = []
         next_step, curr_time = self.__get_next_step(event_times, reaction_times,
                                                     delayed_events,
                                                     model.tspan[-1], next_tau)
         curr_state['t'] = curr_time
 
-        # Update states of all species based on changes made to species through
-        # ODE processes.  This will update all species whose mode is set to
-        # 'continuous', as well as 'dynamic' mode species which have been
-        # flagged as deterministic.
-        for spec_name, species in model.listOfSpecies.items():
-            if not species.constant:
-                curr_state[spec_name] = sol.sol(curr_time)[y_map[spec_name]]
-
         # Stochastic Reactions are also fired through a root-finding method
         # which mirrors the standard SSA probability.  Since we are using
         # Tau-Leaping, rather than firing reactions based on a direct poisson
@@ -486,7 +494,7 @@ class TauHybridSolver(GillesPySolver):
         # added (representing a single-firing each time) until the state value
         # of the reaction is once again negative.
         for rxn in compiled_reactions:
-            curr_state[rxn] = sol.sol(curr_time)[y_map[rxn]]
+            curr_state[rxn] = sol.y[y_map[rxn]]
 
         # In the case that a major change occurs to the system (outside of the
         # standard ODE process) is caused by an event trigger, we then examine
@@ -528,7 +536,7 @@ class TauHybridSolver(GillesPySolver):
         :type save_times: list
         :return curr_state, curr_time, save_times, sol
 
-        sol - Python object returned from solve_ivp which contains all solution
+        sol - Python object returned from LSODA which contains all solution
         data.
         """
 
@@ -585,11 +593,11 @@ class TauHybridSolver(GillesPySolver):
             assignment_state = curr_state.copy()
             for s in range(len(species)):
                 # Get ODE Solutions
-                trajectory[trajectory_index][s + 1] = sol.sol(time)[s]
+                trajectory[trajectory_index][s + 1] = sol.y[s]
                 # Update Assignment Rules for all processed time points
                 if len(model.listOfAssignmentRules):
                     # Copy ODE state for assignments
-                    assignment_state[species[s]] = sol.sol(time)[s]
+                    assignment_state[species[s]] = sol.y[s]
             assignment_state['t'] = time
             for ar in model.listOfAssignmentRules.values():
                 assignment_value = eval(ar.formula, {**eval_globals, **assignment_state})
@@ -764,14 +772,9 @@ class TauHybridSolver(GillesPySolver):
         steps/save points for event detection
         :type event_sensitivity: int
 
-        :param integrator: integrator method to be used form scipy.integrate.solve_ivp. Options include 'RK45', 'RK23',
-        'Radau', 'BDF', and 'LSODA'.
-        For more details, see https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.solve_ivp.html
-        :type integrator: str
-
         :param integrator_options:  contains options to the scipy integrator. by default, this includes
         rtol=1e-9 and atol=1e-12.  for a list of options,
-        see https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.solve_ivp.html.
+        see https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.LSODA.html.
         Example use: {max_step : 0, rtol : .01}
         :type integrator_options: dict
 
