@@ -1,10 +1,9 @@
 import unittest
-from gillespy2.example_models import Example
+from example_models import Example
 from gillespy2.core import Model, StochMLDocument
 from gillespy2.core.gillespyError import *
-from gillespy2.solvers.numpy.basic_ode_solver import BasicODESolver
-from gillespy2.solvers.numpy.ssa_solver import NumPySSASolver
-import numpy as np
+from gillespy2 import ODESolver
+from gillespy2 import NumPySSASolver
 
 
 class TestStochML(unittest.TestCase):
@@ -13,7 +12,7 @@ class TestStochML(unittest.TestCase):
         model = Example()
         stochml = StochMLDocument.from_model(model)
         stochml_model = stochml.to_model('model')
-        stochml_model.run(solver=BasicODESolver)
+        stochml_model.run(solver=ODESolver)
         stochml_model.run(solver=NumPySSASolver)
 
 
