@@ -1,7 +1,8 @@
 import unittest
-from gillespy2.example_models import *
+from example_models import *
 from gillespy2.core.gillespyError import *
-from gillespy2.solvers.numpy.basic_ode_solver import BasicODESolver
+from gillespy2 import ODESolver
+#from TestBattery import *
 
 class TestExampleModels(unittest.TestCase):
 
@@ -11,7 +12,7 @@ class TestExampleModels(unittest.TestCase):
 
     def test_lacOperon_example(self):
         lacOperon_model = LacOperon()
-        results = lacOperon_model.run(solver=BasicODESolver)
+        results = lacOperon_model.run(solver=ODESolver)
 
     def test_schlogl_example(self):
         schlogl_model = Schlogl()
@@ -32,7 +33,9 @@ class TestExampleModels(unittest.TestCase):
     def test_tyson2StateOscillator_example(self):
         tyson2StateOscillator_model = Tyson2StateOscillator()
         results = tyson2StateOscillator_model.run()
-
+    
+    #def test_test_battery(self):
+        #timing_battery(100, 0.1)
 
 if __name__ == '__main__':
     unittest.main()
